@@ -9,7 +9,11 @@ our (@ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 @ISA = qw (Exporter);
 
 @EXPORT = qw (&parse_am_file &output_am_file 
-	      &parse_configure_in &output_configure_in
+	      &parse_configure_in parse_configure_in_buffer
+	      &output_configure_in
+	      &configure_rewrite_variable &configure_remove_variable
+	      &configure_rewrite_packages
+	      &configure_rewrite_package_info
 	      &use_macro &use_rule &expand_one_var
 	      &macro_create &macro_remove &macro_rewrite 
 	      &macro_append_text &macro_prepend_text &macro_remove_text
@@ -146,17 +150,17 @@ sub parse_am_file
 
 	} elsif (/$COMMENT_PATTERN/o) {
 	    if ($continued_line) {
-		&parser_error (101, "$filename:$.: comment following " .
+		&report_error (101, "$filename:$.: comment following " .
 			       "trailing backslash");
 		return {};
 	    };
 
 	} elsif ($continued_line) {
-	    $continued_line = /\\$/;
+	    $continued_line = /\\\s*$/;
 	    if ($in_rule) {
 		if ($rules{$current_rule}{actions} eq "") {
 		    # Strip trailing backslash from rule deps
-		    s/\\w*$//;
+		    s/\\\s*$//;
 		    $rules{$current_rule}{dependencies} .= $_;
 		} else {
 		    $rules{$current_rule}{actions} .= $_;
@@ -165,7 +169,7 @@ sub parse_am_file
 	    elsif ($current_macro) {
 		# Chop newline and backslash if this line is
 		# continued.  ensure trailing whitespace exists.
-		s/\\w*$//;
+		s/\\\s*$//;
 		$macros{$current_macro}{contents} .= $_;
 	    }
 	    else {
@@ -182,7 +186,7 @@ sub parse_am_file
 
 	    my $value = $2;
 	    if (defined ($value)) {
-		$value =~ s/\\w*$//;
+		$value =~ s/\\\s*$//;
 	    } else {
 		$value = "";
 	    };
@@ -192,7 +196,7 @@ sub parse_am_file
 			   used         => 0 };
 	    
 	    &debug ("$filename:$.: Found rule '$1'");
-	    $continued_line = /\\$/;
+	    $continued_line = /\\\s*$/;
 
 	} elsif (($is_ok_macro = /$MACRO_PATTERN/o)
 		 || /$BOGUS_MACRO_PATTERN/o) {
@@ -227,7 +231,7 @@ sub parse_am_file
 
 	    &debug ("$filename:$.: Found macro '$1'");
 	    
-	    $continued_line = /\\$/;
+	    $continued_line = /\\\s*$/;
 
 	} elsif (/$INCLUDE_PATTERN/o) {
 	    # Include
@@ -249,7 +253,7 @@ sub parse_am_file
 	    }
 
 	    $current_macro = "";
-	    $continued_line = /\\$/;
+	    $continued_line = /\\\s*$/;
 	    
 	    if ($in_rule) {
 		$rules{$current_rule}{actions} .= $_;
@@ -384,30 +388,15 @@ sub get_config_files ($)
     };
 }
 
-sub parse_configure_in ($)
+sub parse_configure_in_buffer
 {
-    my $project_dir = shift;
-    my $config_file;
-
-    if (-e "$project_dir/configure.ac") {
-	$config_file = "$project_dir/configure.ac"
-    } elsif (-e "$project_dir/configure.in") {
-	$config_file = "$project_dir/configure.in";
-    } else {
-	&report_error (100, "Neither configure.in nor configure.ac exist");
-	return {};
-    };
-
-    my $contents = &read_entire_file ($config_file);
-    if (! $contents) {
-	return {};
-    };
-
+    my ($config_file, $contents) = @_;
+    
     # get configured files, either from AC_OUTPUT or from AC_CONFIG_FILES
     my (undef, $output) = &get_config_files ($contents);
 
     # get substed vars
-    my (%vars, %tmpvars);
+    my (%vars, %tmpvars, @vars_order);
     my @tmp = &extract_syntactic_arguments ($contents, "AC_SUBST\\(", "\\(", "\\)");
     # catch variable definitions
     while ($contents =~ m/(\w+)=(.+)$/mg) {
@@ -419,6 +408,7 @@ sub parse_configure_in ($)
 	my (undef, $varname) = @$tmp;
 	if (defined ($tmpvars{$varname})) {
 	    $vars{$varname} = $tmpvars{$varname};
+	    push (@vars_order, $varname);
 	};
     };
     
@@ -444,17 +434,103 @@ sub parse_configure_in ($)
 	
 	$other_files{$gen} = $input;
     };
-
+    
     my $configure_in = { filename          => $config_file,
 			 contents          => $contents,
 			 other_files       => \%other_files,
 			 vars              => \%vars,
+			 vars_order        => \@vars_order,
 			 dirty             => 0 };
 
     return $configure_in;
 }
 
+sub parse_configure_in ($)
+{
+    my $project_dir = shift;
+    my $config_file;
 
+    if (-e "$project_dir/configure.ac") {
+	$config_file = "$project_dir/configure.ac"
+    } elsif (-e "$project_dir/configure.in") {
+	$config_file = "$project_dir/configure.in";
+    } else {
+	&report_error (100, "Neither configure.in nor configure.ac exist");
+	return {};
+    };
+
+    my $contents = &read_entire_file ($config_file);
+    if (! $contents) {
+	return {};
+    };
+    return parse_configure_in_buffer ($config_file, $contents);
+}
+
+sub configure_rewrite_packages
+{
+    my ($configure_in, $module, $packages) = @_;
+
+    $packages =~ s/,/ /sg;
+    $packages =~ s/ +/ /sg;
+    # Module is defined.
+    if ($configure_in->{'contents'} =~ s/(PKG_CHECK_MODULES\([\s\[]*$module[\s\]]*,[\s\[]*)[^\,\)\]]+(.*?\))/$1$packages$2/sm) {
+    } else {
+	## Could not find any proper place to put the new variable.
+	## Just putting it on top.
+	$configure_in->{'contents'} =~ s/AC_OUTPUT/PKG_CHECK_MODULES($module, $packages)\n\nAC_OUTPUT/sm;
+    }
+    $configure_in->{"dirty"} = 1;
+}
+
+sub configure_rewrite_variable
+{
+    my ($configure_in, $var, $value) = @_;
+    
+    if ($configure_in->{'contents'} =~ /^\Q$var\E=/m) {
+	$configure_in->{'contents'} =~
+	    s/^\Q$var\E=.*$/$var=$value/m;
+    } elsif ($configure_in->{'contents'} =~ /^AC_INIT\(.*?\)\s*$/m) {
+	$configure_in->{'contents'} =~
+	    s/^(AC_INIT\(.*?\))\s*$/$1\n\n$var=$value\nAC_SUBST($var)\n/m;
+    } else {
+	## Could not find any proper place to put the new variable.
+	## Just putting it on top.
+	$configure_in->{'contents'} =
+	    "$var=$value\nAC_SUBST($var)\n". $configure_in->{'contents'};
+    }
+    $configure_in->{"dirty"} = 1;
+}
+
+sub configure_remove_variable
+{
+    my ($configure_in, $var) = @_;
+    if ($configure_in->{'contents'} =~ /^\Q$var\E=/m)
+    {
+	$configure_in->{'contents'} =~
+	    s/^\Q$var\E=.*$//m;
+	$configure_in->{dirty} = 1;
+    }
+}
+
+sub configure_rewrite_package_info
+{
+    my ($configure_in, $package_name, $package_version, $package_url) = @_;
+    
+    if ($package_url ne "")
+    {
+	$configure_in->{contents} =~ s/AC_INIT\(.*?\)/AC_INIT($package_name, $package_version, $package_url)/sm;
+    }
+    elsif ($package_version ne "")
+    {
+	$configure_in->{contents} =~ s/AC_INIT\(.*?\)/AC_INIT($package_name, $package_version)/sm;
+    }
+    elsif ($package_name ne "")
+    {
+	$configure_in->{contents} =~ s/AC_INIT\(.*?\)/AC_INIT($package_name)/sm;
+    }    
+    
+    $configure_in->{dirty} = 1;
+}
 
 ######################################################################
 # Direct file modification functions
@@ -662,7 +738,7 @@ sub analyze_macro_style
 	my $line_copy = &expand_tabs ($line);
 	my $bs = $bs_column;
 	
-	if ($line_copy =~ /\\$/) {
+	if ($line_copy =~ /\\\s*$/) {
 	    # first get the column of the backslash
 	    $bs = rindex $line_copy, '\\';
 	};
@@ -674,7 +750,7 @@ sub analyze_macro_style
 	    $line =~ /^(\s+)\S+/;
 	    $leading_space = $1;
 	};
-	$line_copy =~ s/\\$//;
+	$line_copy =~ s/\\\s*$//;
 	
 	my @items = split /\s+/, &trim ($line_copy);
 	
@@ -698,7 +774,7 @@ sub analyze_macro_style
 	};
 	
 	$first_line = 0;
-    } while ($line =~ /\\$/);
+    } while ($line =~ /\\\s*$/);
     
     &debug ("Macro: $macro_name, " .
 	    "bs_column = $bs_column, items_per_line = $items_per_line, ".
@@ -850,7 +926,7 @@ sub macro_prepend_text
 	$first_line =~ /$MACRO_PATTERN/;
 	my $leading_text = $3;
 	$text .= " " . $leading_text;
-	$text =~ s/\\$//;
+	$text =~ s/\\\s*$//;
 	$first_line = substr $first_line, 0, -length ($leading_text) - 1;
 
 	my @lines = &format_macro ($first_line, $text, $macro_style, 
@@ -944,7 +1020,7 @@ sub macro_remove
 	    &debug ("Removing macro $macro_name at line $line");
 	    
 	    my $count = 1;
-	    while (@{$makefile->{lines}}[$line + $count - 2] =~ /\\$/) {
+	    while (@{$makefile->{lines}}[$line + $count - 2] =~ /\\\s*$/) {
 		$count++;
 	    };
 	    
@@ -959,12 +1035,15 @@ sub macro_rewrite
 {
     # Rewrites the macro with the new text, but trying to accomodate to the
     # previous style
-    my ($makefile, $macro_name, $text) = @_;
+    my ($makefile, $macro_name, $text, $hint_near) = @_;
 
     my $macro = $makefile->{macros}{$macro_name};
     if (!defined ($macro)) {
+    	if ($hint_near eq "") {
+	    $hint_near = &last_macro ($makefile);
+	}
 	# If the macro doesn't exist create it
-	&macro_create ($makefile, $macro_name, $text, &last_macro ($makefile));
+	&macro_create ($makefile, $macro_name, $text, $hint_near);
 
     } elsif ($macro->{atline} > 0) {
 	my ($last_line_num, $macro_style) =
@@ -1020,7 +1099,7 @@ sub macro_remove_text
 	    } else {
 		# else, just remove the text, the trailing bs and reformat the line
 		$line =~ s/$remove_text\s//;
-		$line =~ s/\\$//;
+		$line =~ s/\\\s*$//;
 		if ($line !~ /^\s+/ && $line_num > $macro->{atline}) {
 		    $line = $macro_style->{leading_space} . $line;
 		};
@@ -1071,7 +1150,7 @@ sub rule_remove
 	    my $maxcount = scalar @{$makefile->{lines}} - $line;
 	    # count dependency lines
 	    while ($count < $maxcount && 
-		   @{$makefile->{lines}}[$line + $count - 2] =~ /\\$/) {
+		   @{$makefile->{lines}}[$line + $count - 2] =~ /\\\s*$/) {
 		$count++;
 	    };
 	    # count action lines
@@ -1223,7 +1302,7 @@ sub analyze_text ($$$)
 
 	# sample data
 	$widths[$line_length]++;
-	$backslashed = 1 if $expanded_line =~ m/\\$/;
+	$backslashed = 1 if $expanded_line =~ m/\\\s*$/;
 	$expanded_line = chop_backslash $expanded_line;
 	if ($line_number != 0 && $line_number != @lines - 1) {
 	    $expanded_line =~ m/^(\s*)\S.*\S(\s*)$/;
@@ -1419,8 +1498,7 @@ sub edit_config_files ($$$)
 					 $operand, $style);
 
     } elsif ($action eq "remove") {
-	# FIXME
-
+	$new_config_files =~ s/\Q$operand\E\s*//g;
     };
 
     # save result string
